#extension GL_OES_EGL_image_external : require

precision lowp float;
precision mediump int;

uniform samplerExternalOES mOESTexture;
uniform sampler2D mTexture;
uniform int mKernel[100];

uniform vec4 mConfig;
uniform int mBlurEnabled;
uniform vec4 mBlurBoundary;
uniform vec2 mTexBoundary;
uniform mat4 mLookupMatrix;

varying vec2 mTexCord;
varying float mLogoCoef;
varying float mBlackEdgeCoef;

void main() {
    float isVertical = mConfig.w;

    // 片头logo上下黑边则直接返回黑色
    if (mLogoCoef == 1.0) {
        gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
        return;
    }

    // 屏幕中间的MV
    if (mTexCord.y >= mBlurBoundary.s && mTexCord.y <= mBlurBoundary.t) {
        if (isVertical==0.0) {  // 第一次绘制
            gl_FragColor = texture2D(mOESTexture, mTexCord);
        } else {                // 第二次绘制
            gl_FragColor = texture2D(mTexture, mTexCord);
        }
        return;
    }

    // 视频上下黑边则直接返回，不作高斯模糊
    if (mBlackEdgeCoef == 1.0 || mBlurEnabled == 0) {
        gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
        return;
    }

    // 渐变模糊系数[0, 1]
    // 顶部高斯区域
    float k1 = smoothstep(mBlurBoundary.s+0.001/*加极小值以避免两个边界相等时，渐变方向被改变*/, mBlurBoundary.p, mTexCord.y);
    // 底部高斯区域
    float k2 = smoothstep(mBlurBoundary.t-0.001/*减极小值以避免两个边界相等时，渐变方向被改变*/, mBlurBoundary.q, mTexCord.y);
    float k = k1 + k2;

    int radius = int(mConfig.x);
    int square = radius*2+1;
    float radiusF = float(radius);
    float stepX = mConfig.y;
    float stepY = mConfig.z;
    float stepKernal = 100.0/float(square);

    vec4 rst=vec4(0.0);
    float num=0.0;

    float startX = mTexCord.x-stepX*radiusF;
    float startY = mTexCord.y-stepY*radiusF;
    float kernalX = 0.0;
    float kernalY = 0.0;

    if (isVertical==0.0){
        //垂直采样(第一次)
        for (int i=0;i<square;i++){
            float texCordX = mTexCord.x;
            float texCordY = startY;
            if (startY < mTexBoundary.s) {
                // (x1 + x2) / 2 = x0，x0为镜像中心点，x1与x2为镜像对称点
                texCordY = 2.0 * mTexBoundary.s - startY;
                // 向上稍许移动，避开视频上下的黑边
                texCordY += 0.2;
            } else if (startY > mTexBoundary.t) {
                // 底部高斯模糊区域放大2倍，避免上下黑边视频的底部出现大面积黑色背景
                texCordX *= 0.5;
                texCordY = 2.0 * mTexBoundary.t - (startY+1.3) * 0.5;
            }

            float tempKernal = float(mKernel[int(kernalY)]);
            rst += texture2D(mOESTexture, vec2(texCordX, texCordY)) * tempKernal;
            num += tempKernal;
            kernalY+=stepKernal;
            startY+=stepY;
        }

        gl_FragColor = rst/num;
    } else {
        //水平采样(第二次)
        for (int i=0;i<square;i++){
            float tempKernal = float(mKernel[int(kernalX)]);
            rst += texture2D(mTexture, vec2(startX, mTexCord.y)) * tempKernal;
            num += tempKernal;
            kernalX+=stepKernal;
            startX+=stepX;
        }

        // 从初始图片中获取像素值，再与模糊值进行渐变混合
        vec4 blurColor = rst/num;
        vec4 oriTexCord = mLookupMatrix * vec4(mTexCord, 0.0, 1.0);
        vec4 oriColor = texture2D(mOESTexture, oriTexCord.xy);
        // k*blurColor + (1-k)*oriColor
        gl_FragColor = mix(oriColor, blurColor, k);
    }
}
