#version 100

//在顶点着色阶段，
//如果没有用户自定义的默认精度，
//那么 int 和 float 都默认为 highp 级别；
//而在片元着色阶段，如果没有用户自定义的默认精度，
//没有默认精度了，需在每个变量前放置精度描述符
precision highp float;
precision mediump int;

uniform sampler2D mOriTexture;
uniform sampler2D mTexture;
uniform int mKernel[100];

uniform vec4 mConfig;
uniform vec4 mBlurBoundary;
uniform vec2 mTexBoundary;
uniform mat4 mLookupMatrix;

varying vec2 mTexCord;

void main() {
    // 屏幕中间的MV
    if (mTexCord.y >= mBlurBoundary.s && mTexCord.y <= mBlurBoundary.t) {
        gl_FragColor = texture2D(mTexture, mTexCord);
        return;
    }

    // 渐变模糊系数[0, 1]
    // 参考smoothstep平滑曲线，https://blog.csdn.net/u010333737/article/details/82859246
    float k1 = smoothstep(mBlurBoundary.s, mBlurBoundary.p, mTexCord.y);
    float k2 = smoothstep(mBlurBoundary.t, mBlurBoundary.q, mTexCord.y);
    float k = k1 + k2;

    int radius = int(mConfig.x);
    int square = radius*2+1;
    float radiusF = float(radius);
    float stepX = mConfig.y;
    float stepY = mConfig.z;
    float isVertical = mConfig.w;
    float stepKernal = 100.0/float(square);

    vec4 rst=vec4(0.0);
    float num=0.0;

    float startX = mTexCord.x-stepX*radiusF;
    float startY = mTexCord.y-stepY*radiusF;
    float kernalX = 0.0;
    float kernalY = 0.0;

    if (isVertical==0.0){
        //垂直采样
        for (int i=0;i<square;i++) {
            float texCordY = startY;
            if (startY < mTexBoundary.s) {
                // (x1 + x2) / 2 = x0，x0为镜像中心点，x1与x2为镜像对称点
                texCordY = 2.0 * mTexBoundary.s - startY;
            } else if (startY > mTexBoundary.t) {
                texCordY = 2.0 * mTexBoundary.t - startY;
            }

            float tempKernal = float(mKernel[int(kernalY)]);
            rst += texture2D(mTexture, vec2(mTexCord.x, texCordY)) * tempKernal;
            num += tempKernal;
            kernalY+=stepKernal;
            startY+=stepY;
        }

        gl_FragColor = rst/num;
    } else {
        //水平采样(第二次)
        for (int i=0;i<square;i++){
            float tempKernal = float(mKernel[int(kernalX)]);
            rst += texture2D(mTexture, vec2(startX, mTexCord.y)) * tempKernal;
            num += tempKernal;
            kernalX+=stepKernal;
            startX+=stepX;
        }

        // 从初始图片中获取像素值，再与模糊值进行渐变混合
        vec4 blurColor = rst/num;
        vec4 oriTexCord = mLookupMatrix * vec4(mTexCord, 0.0, 1.0);
        vec4 oriColor = texture2D(mOriTexture, oriTexCord.xy);
        // k*blurColor + (1-k)*oriColor
        gl_FragColor = mix(oriColor, blurColor, k);
    }
}
