#extension GL_OES_EGL_image_external : require
// int 和 float 默认为 highp 级别

uniform mat4 mMatrix;
uniform mat4 mSTMatrix;
uniform samplerExternalOES mOESTexture;
uniform int mDetectLogo;
uniform int mDetectBlackEdge;

attribute vec2 mPosAttr;
attribute vec2 mTexCordAttr;

varying vec2 mTexCord;
varying float mLogoCoef;
varying float mBlackEdgeCoef;

bool isLogo() {
    // 片头logo上下黑边则直接返回黑色
    float avgGray = 0.0;
    int numPoints = 20;
    float step = 1.0 / float(numPoints);
    for (int i=0; i<numPoints; i++) {
        // 纹理最下方采集5个点的灰度值，平均后小于一定阈值则认为是黑色
        vec3 cb = texture2D(mOESTexture, vec2(0.2, float(i) * step)).rgb;
        vec3 ct = texture2D(mOESTexture, vec2(0.8, float(i) * step)).rgb;
        avgGray += cb.r*.299 + cb.g*.587 + cb.b*.114;
        avgGray += ct.r*.299 + ct.g*.587 + ct.b*.114;
    }
    avgGray /= float(numPoints) * 2.0;

    // 中间的咪咕红
//    vec3 cc = texture2D(mOESTexture, vec2(0.4, 0.5)).rgb;
//    float grayC = cc.r*.299 + cc.g*.587 + cc.b*.114;
//    bool isMiguRed = cc.r > 0.8 && abs(cc.g - 0.31) < 0.2 && abs(cc.b - 0.46) < 0.2;

    return avgGray < 0.02/* && isMiguRed*/;
}

bool isBlackEdge() {
    // 视频上下黑边则直接返回
    float avgGrayB = 0.0;
    float avgGrayT = 0.0;
    float span = 1.0 / 64.0;
    for (int i=0; i < 64; i++) {
        // 纹理最下方采集若干点的灰度值，平均后小于一定阈值则认为是黑色
        vec3 cb = texture2D(mOESTexture, vec2(float(i) * span, 0.03)).rgb;
        vec3 ct = texture2D(mOESTexture, vec2(float(i) * span, 0.01)).rgb;
        avgGrayB += cb.r*.299 + cb.g*.587 + cb.b*.114;
        avgGrayT += ct.r*.299 + ct.g*.587 + ct.b*.114;
    }
    avgGrayB /= 64.0;
    avgGrayT /= 64.0;
    return avgGrayB < 0.005 && avgGrayT < 0.005;
}

void main() {
    mTexCord = (mSTMatrix * vec4(mTexCordAttr, 0.0, 1.0)).xy;

    // LOGO检测与黑边检测放在vertex shader中，虽然多个顶点是重复计算，但相对于fragment shader，重复计数次数少得多
    mLogoCoef = (mDetectLogo == 1 && isLogo()) ? 1.0 : 0.0;
    mBlackEdgeCoef = (mDetectBlackEdge == 1 && isBlackEdge()) ? 1.0 : 0.0;

    gl_Position = mMatrix * vec4(mPosAttr, 0.0, 1.0);
}